# Generated by roxygen2: do not edit by hand

S3method(summary,cxr_er_fit)
S3method(summary,cxr_pm_fit)
S3method(summary,cxr_pm_multifit)
export(BH_er_lambdacov_global_effectcov_global_responsecov_global)
export(BH_er_lambdacov_none_effectcov_none_responsecov_none)
export(BH_pm_alpha_global_lambdacov_none_alphacov_none)
export(BH_pm_alpha_none_lambdacov_none_alphacov_none)
export(BH_pm_alpha_pairwise_lambdacov_global_alphacov_global)
export(BH_pm_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(BH_pm_alpha_pairwise_lambdacov_none_alphacov_none)
export(BH_project_alpha_global_lambdacov_none_alphacov_none)
export(BH_project_alpha_none_lambdacov_none_alphacov_none)
export(BH_project_alpha_pairwise_lambdacov_global_alphacov_global)
export(BH_project_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(BH_project_alpha_pairwise_lambdacov_none_alphacov_none)
export(LV_er_lambdacov_global_effectcov_global_responsecov_global)
export(LV_er_lambdacov_none_effectcov_none_responsecov_none)
export(LV_pm_alpha_global_lambdacov_none_alphacov_none)
export(LV_pm_alpha_none_lambdacov_none_alphacov_none)
export(LV_pm_alpha_pairwise_lambdacov_global_alphacov_global)
export(LV_pm_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(LV_pm_alpha_pairwise_lambdacov_none_alphacov_none)
export(LV_project_alpha_global_lambdacov_none_alphacov_none)
export(LV_project_alpha_none_lambdacov_none_alphacov_none)
export(LV_project_alpha_pairwise_lambdacov_global_alphacov_global)
export(LV_project_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(LV_project_alpha_pairwise_lambdacov_none_alphacov_none)
export(LW_er_lambdacov_global_effectcov_global_responsecov_global)
export(LW_er_lambdacov_none_effectcov_none_responsecov_none)
export(LW_pm_alpha_global_lambdacov_none_alphacov_none)
export(LW_pm_alpha_none_lambdacov_none_alphacov_none)
export(LW_pm_alpha_pairwise_lambdacov_global_alphacov_global)
export(LW_pm_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(LW_pm_alpha_pairwise_lambdacov_none_alphacov_none)
export(LW_project_alpha_global_lambdacov_none_alphacov_none)
export(LW_project_alpha_none_lambdacov_none_alphacov_none)
export(LW_project_alpha_pairwise_lambdacov_global_alphacov_global)
export(LW_project_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(LW_project_alpha_pairwise_lambdacov_none_alphacov_none)
export(RK_er_lambdacov_global_effectcov_global_responsecov_global)
export(RK_er_lambdacov_none_effectcov_none_responsecov_none)
export(RK_pm_alpha_global_lambdacov_none_alphacov_none)
export(RK_pm_alpha_none_lambdacov_none_alphacov_none)
export(RK_pm_alpha_pairwise_lambdacov_global_alphacov_global)
export(RK_pm_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(RK_pm_alpha_pairwise_lambdacov_none_alphacov_none)
export(RK_project_alpha_global_lambdacov_none_alphacov_none)
export(RK_project_alpha_none_lambdacov_none_alphacov_none)
export(RK_project_alpha_pairwise_lambdacov_global_alphacov_global)
export(RK_project_alpha_pairwise_lambdacov_global_alphacov_pairwise)
export(RK_project_alpha_pairwise_lambdacov_none_alphacov_none)
export(abundance_projection)
export(avg_fitness_diff)
export(build_param)
export(calculate_densities)
export(competitive_ability)
export(cxr_er_bootstrap)
export(cxr_er_fit)
export(cxr_generate_test_data)
export(cxr_pm_bootstrap)
export(cxr_pm_fit)
export(cxr_pm_multifit)
export(densities_to_df)
export(fill_demography_matrix)
export(fill_dispersal_matrix)
export(fill_transition_matrix)
export(fitness_ratio)
export(generate_vital_rate_coefs)
export(niche_overlap)
export(species_fitness)
export(vec_permutation_matrices)
export(vital_rate)
import(stats)
importFrom(utils,combn)
