- [x] 上書き防止。既にファイルが存在する場合は、上書きするかどうかを確認する。(y/N)
- [x] 作成時にディレクトリを指定するオプションを追加する。
- [x] 変数にデフォルト値を設定する
- [x] 変数を空白のときにエラーを表示する (デフォルト値が設定されていない場合)
- [x] ドライラン
- [x] force/backup/skipで上書きの挙動を指定
- [x] テンプレ検索、fzfで選択してテンプレートを指定する
- [x] pre/post hookの追加
- [x] バリデーション/lintの追加
